package PureMessage::Policy::Delay;

use strict;
use warnings;
use PureMessage::Policy::Delay::Redis::Config qw(%redis_config);
use PureMessage::Version qw($PREFIX);
use PureMessage::_sys;
use Redis;
# my ($ip_port, $logger) = (undef,undef);
my $ip_port = undef;
my $HDB_IS_UP = 0;
my $shdb_client = undef;
connect_shdb();

BEGIN {
	require Exporter;
	our @ISA = qw(Exporter);
	our @EXPORT_OK = qw(add_sender_history_header add_rescan_header update_sender_history_counters);

	$ip_port = $redis_config{server_ip}.":".$redis_config{port};
	# use PureMessage::Logger qw(LOG_WARNING LOG_DEBUG LOG_NOTICE LOG_ERROR);
	# $logger = PureMessage::Logger->new( ident => 'DelayQueue' );
}
# connect to sender history database
sub connect_shdb {
	return $shdb_client if $shdb_client;
	my $shdb_server_ip   = $redis_config{server_ip};
	my $shdb_server_port = $redis_config{port};
	my $shdb_id = 0;
    my %redis_obj=(
		server    => "$shdb_server_ip:$shdb_server_port",
		name      => PureMessage::_sys::gethostname() . ":$$",
		reconnect => 3,
		every     => 1.5 * 1000000,
		conservative_reconnect => 1,
		no_auto_connect_on_new => 1,
		cnx_timeout            => 15,
	);
	if (defined $redis_config{password} ){
		$redis_obj{password}=  $redis_config{password};
	}		
	$shdb_client = Redis->new(%redis_obj);

	eval {
		$shdb_client->connect;
		$HDB_IS_UP = 1;
	};
	if ($@) {
		$HDB_IS_UP = 0;
		$shdb_client = undef;
	}
	return $shdb_client;
}

sub add_sender_history_header {
	my $me = "add_sender_history_header";
	my $msg    = shift;
	unless ( $HDB_IS_UP ) {
		$msg->log('ERROR', "$me: Unable to connect to Sender History Database server at $ip_port. Delay queue protection will not work.");
		return 0;
	}

	my $header = undef;
	my $ip = $msg->minfo_value("Fur");
	# 'Fur' can be 'none', an IPv4 or IPv6 address
	if($ip eq 'none') {
		$msg->log('ERROR', "$me: No Fur (first untrusted relay) value present!\n");
		return 0;
	}

	# Get db_age
	my $db_age = undef;
	eval { $db_age = $shdb_client->get("db_age"); };
	if($@) {
		$msg->log('ERROR', "$me: error querying sender history database for IP '$ip' at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('get', $error_msg);
		return 0;
	}
	
	my %data = ();
	eval {
		my $tmp = $shdb_client->HGETALL($ip);
		%data = @{$tmp} if ( $tmp && ( ref($tmp) =~ /ARRAY/ ) && scalar(@{$tmp}) );
	};
	if($@) {
		$msg->log('ERROR', "$me: error querying sender history database for IP '$ip' at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('hmget', $error_msg);
		return 0;
	}

	my $now = time();
	if ( $db_age && %data ) {
		# update ls and (if needed) fs (after reset, fs needs to be set to the time of first mail arrival)
		eval {
			$shdb_client->hmset( $ip, 'ls', $now );
			unless ($data{fs}) {
				$shdb_client->hmset( $ip, 'fs', $now );
				$data{fs} = $now;
			}
		};
		if($@) {
			$msg->log('ERROR', "$me: error updating last/first seen value in sender history database for IP '$ip' at $ip_port\n");
			my $error_msg = chomp($@);
			redis_log('hmset', $error_msg);
			return 0;
		}
		
		# calculate values of header fields and add header
		my $da = $now - $db_age;
		my $fs = $now - $data{fs};
		my $hc = $data{mc} - $data{sc};
		my $sp = ( $data{mc} == 0 ) ? 0 : int(($data{sc}/$data{mc})*100);
		my $fso = $now - $data{fso};
		my $re = $data{nr};
		
		my $sd = $data{sd};
		my $hd = $data{hd};
		$msg->header_add('X-Sophos-SenderHistory', "ip=$ip,fs=$fs,da=$da,mc="
					.$data{mc}.",sc=".$data{sc}.",hc=$hc,sp=$sp,fso=$fso,re=".$data{nr}.",sd=".$data{sd}.",hd=".$data{hd});
		return 1;
	}
	
	unless ($db_age) {
		eval { $shdb_client->set( 'db_age', $now ); };
		if($@) {
			$msg->log('ERROR', "$me: error setting the value of db_age in sedner history database at $ip_port\n");
			my $error_msg = chomp($@);
			redis_log('set', $error_msg);
			return 0;
		}
	}

    eval {
		$shdb_client->hmset( $ip, 'ls', $now, 'fso', $now, 'fs', $now, 'mc',0, 'mc_today',0, 'sc',0, 'sc_today',0, 'nr',0, 'sd_hd_str',0, 'sd',0, 'hd',0);
	};
	if($@) {
		$msg->log('ERROR', "$me: error inserting record of ip '$ip' in sender history database at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('hmset', $error_msg);
		return 0;
	}

	return 1;
}

sub add_rescan_header {
	my $me = "add_rescan_header";
	my $msg = shift;
	unless ( $HDB_IS_UP ) {
		$msg->log('ERROR', "$me: Unable to connect to Sender History Database server at $ip_port. Delay queue protection will not work.");
		return 0;
	}

	my $sender_history_header = $msg->header('X-Sophos-SenderHistory');
	unless ($sender_history_header) {
		$msg->log('ERROR', "$me: no 'X-Sophos-SenderHistory' header present in a delayed mail!\n");
		return 0;
	}
	
	my @temp = split(',',$sender_history_header);
	my %data = map{ split('=',$_) } @temp;

	my %db_data = ();
	eval {
		my $tmp = $shdb_client->HGETALL($data{ip});
		%db_data = @{$tmp} if ( $tmp && ( ref($tmp) =~ /ARRAY/ ) && scalar(@{$tmp}) );
	};
	if($@) {
		$msg->log('ERROR', "$me: error querying sender history database for IP ".$data{ip}." at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('hmget', $error_msg);
		return 0;
	}

	unless(%db_data) {
		$msg->log('ERROR', "$me: expected record not found in sender history database for IP ".$data{ip}." at $ip_port\n");
		return 0;
	}
	
	# Get db_age
	my $db_age = undef;
	eval { $db_age = $shdb_client->get("db_age"); };
	if($@) {
		$msg->log('ERROR', "$me: no entry for db_age in Sendor History Database at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('get', $error_msg);
		return 0;
	}
	
	my $now = time();
	# calculate values of header fields and add header
	my $fs = $now - $db_data{fs};
	my $da = $now - $db_age;
	my $mc = $db_data{mc} - $data{mc};
	my $sc = $db_data{sc} - $data{sc};
	my $hc = ( $db_data{mc} - $db_data{sc} ) - $data{hc};
	my $sp = $db_data{mc} == 0 ? 0 : int(($db_data{sc}/$db_data{mc})*100);
	my $fso = $now - $db_data{fso};
	my $re = $db_data{nr} - $data{re};
	my $sd = $db_data{sd};
	my $hd = $db_data{hd};
	$msg->header_add('X-Sophos-ReScan', "ip=".$data{ip}.",fs=$fs,da=$da,mc=$mc,sc=$sc,hc=$hc,sp=$sp,fso=$fso,re=$re,sd=$sd,hd=$hd");
	return 1;
}

sub update_sender_history_counters {
	my $me = "update_sender_history_counters";
	my $msg = shift;
	unless ( $HDB_IS_UP ) {
		$msg->log('ERROR', "$me: Unable to connect to Sender History Database server at $ip_port. Delay queue protection will not work.");
		return 0;
	}

	my $spam_probability = shift;
	my $ip = $msg->minfo_value("Fur");
	if($ip eq 'none') {
		$msg->log('ERROR', "$me: No Fur (first untrusted relay) value present!\n");
		return 0;
	}

	eval {
		my $result = $shdb_client->exists($ip);
		unless ($result) {
			$msg->log('ERROR', "$me: expected record not found in sender history database for IP '$ip' at $ip_port\n");
			return 0;
		}
	};
	if($@) {
		$msg->log('ERROR', "$me: error querying sender history database for IP '$ip' at $ip_port\n");
		my $error_msg = chomp($@);
		redis_log('exists', $error_msg);
		return 0;
	}
	
	eval {
		$shdb_client->multi();
		$shdb_client->hincrby($ip, 'mc', '1');
		$shdb_client->hincrby($ip, 'mc_today', '1');
		if($spam_probability > 50) {
			$shdb_client->hincrby($ip, 'sc', '1');
			$shdb_client->hincrby($ip, 'sc_today', '1');
		}
		$shdb_client->exec();
	};
	if ($@) {
		$msg->log('ERROR', "$me: failed to queue the command while updating sender history database for IP '$ip' at $ip_port\n");
		return 0;
	}
	return 1;
}

sub redis_log {
	my ( $method, $error_msg) = @_;
	open(my $log, '>>', "$PREFIX/var/log/redis.log");
	print $log "WARNING: Unable to execute redis command $method\nerror: $error_msg\n";
	close($log);
}

1;
