package PureMessage::Policy::Delay::Config;

use strict;
use warnings;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = '%config';
our %config;

# Validation rules and default values.
my %conf = (
	delay_status          => ['#enum On Off Collect', 'Collect' ],
	delay_min_minutes     => ['#num', 10 ],
	delay_max_minutes     => ['#num', 60 ],
	delay_max_size        => ['#num', 1024 ],
	send_delayed_mail_to  => ['#str', '127.0.0.1:10025' ],
	debug_level	      => ['#num', 100 ],
);

refresh();

sub new {
    my $self = shift;
    my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Policy::Delay::Config->validate($p);
    $p->parse_file(expand_filename("etc/delay.conf"));
    $p->as_hash;
    $p;
}

sub refresh {
    %config = new(undef)->as_hash;
}

sub validate {
    my ($pkg, $p) = @_;

    # Add our validation rules to the parser.
    foreach my $key (keys %conf) {
        my ($type, $def) = @{$conf{$key}};
        $p->typedecl( $key, $type ) if (defined $type);
        $p->default(  $key, $def )  if (defined $def);
    }
}

1;
