package PureMessage::Policy::Delay::DBEntriesService;
use strict;
use warnings;
use Carp;
use ActiveState::Handy qw(run);
use ActiveState::Unix::ProcInfo qw(proc_info);
use PureMessage::Version qw($PREFIX);
use PureMessage::Config qw(expand_filename);

use base 'Exporter';
our @EXPORT_OK = qw(
	dq_db_entries_pid
	dq_db_entries_is_running

	dq_db_entries_stop
	dq_db_entries_start
	dq_db_entries_restart
	dq_db_entries_status
);

my $pidfile = "$PREFIX/var/run/delay_db_entries.pid";
sub dq_db_entries_pid{
	my $pid_file = expand_filename($pidfile);
	my $pid = 0;
	return $pid unless -e $pid_file;
	if(open(PF, '<', $pid_file)){
		$pid = <PF>;
		chomp($pid);
		close PF;
	}
	return $pid;
}

sub dq_db_entries_is_running{
	my $pid = dq_db_entries_pid();
	return 0 unless $pid;
	my $info = proc_info ('root_pid' => $pid);
	return 0 unless $info && $$info{$pid}; 
	return 1;
}

sub dq_db_entries_start{
    return 1 if dq_db_entries_is_running();

    my $cmd  = expand_filename('bin/pmx-dq-entries');
    unless( -x $cmd) {
        croak('pmx-dq-entries program does not have permissions to run.');
    }

    my $rc = 1;
    eval {$rc = run("\@$cmd >/dev/null 2>&1"); };
	return 0 if ($rc || $@);

    # Poll for pmx-dq-entries to start.
    my $count = 10;
    while ($count--) {
        sleep(1);
        if (dq_db_entries_is_running()) {
          return 1;
        }
    }

    return 0;
}

sub dq_db_entries_stop{
	return 1 unless dq_db_entries_is_running();
	my $pid = dq_db_entries_pid();
	my $count = 30;
	if(kill(0, $pid)){
		kill('TERM', $pid);
		while($count and kill(0,$pid)) {
			$count--;  
			sleep(1);
		}
		unlink $pidfile;
		return $count;
	}
	return 0;
}

sub dq_db_entries_restart{
  my $c = 1;
  if(dq_db_entries_is_running()){
    $c = dq_db_entries_stop();
  }
  return 0 unless $c;
  return dq_db_entries_start();
}

sub dq_db_entries_status{
	if (dq_db_entries_is_running()) {
		print "pmx-dq-entries:\n";
		print "\tpid :     ".dq_db_entries_pid()."\n";
		return 1;
	} else {
		print "pmx-dq-entries is not running.\n";
		return 0;
	}
}

1;


