package PureMessage::Policy::Delay::DelayRetry;
use strict;
use warnings;

use File::Find;
use File::Copy;

use Sophos::Config qw(:path);
use PureMessage::Version qw($PREFIX);
use PureMessage::Logger qw(LOG_ERR LOG_DEBUG LOG_INFO);
use PureMessage::Policy::Delay::DelayqPmdb;
my $qdirpath = "$PREFIX/var/qdir";

our @ISA = qw( Exporter );
our @EXPORT_OK = qw(
    delay_queue_retry delay_queue_delete
);

our @EXPORT = qw(
    delay_queue_retry delay_queue_delete
);

our $mail_directory  = "$PREFIX/var/qdir";
our $delay_queue_dir= "$mail_directory/delay";
my $default_log_file = "$PREFIX/var/log/delay.log";

my $logger = PureMessage::Logger->new(
    ident => 'delay_queue_runner',
    log_to => $default_log_file
);


sub set_delay_queue_size {
    my ( $size_to_release, $messages_to_release ) = @_;
    return unless $size_to_release;

    my $delay_db_conf = Sophos::Config->new;
    $delay_db_conf->parse_file(
	"$delay_queue_dir/delay_queue_size",
	create   => 1,
	writable => 1
    );

    path_value(
	$delay_db_conf->root, "disk_used",
	path_value($delay_db_conf->root, "disk_used") - $size_to_release
    );

    path_value(
      $delay_db_conf->root, "message_count",
      path_value($delay_db_conf->root, "message_count") - $messages_to_release
    );

    $delay_db_conf->save;
}

#$id means file name in minute directory.
sub delay_queue_retry_do{
    my $dir       = shift;
    my $filename  = shift;
    my $filepath  = shift;

    if ( -f "$filepath"){
	#NameSample:/\d*_([frt])_([0-9]{1,2})$/
	my $now_min=(localtime(time))[1];
	my $dstfname=sprintf("%d_r_%02d",$filename,($now_min < $dir? ($dir - $now_min) : ($dir+60 -$now_min) ));
	if (move("$filepath", "$delay_queue_dir/rescan/$dstfname")){
	    return 1;
        }
        return 0; #there was error
    }else{
        return 0;
    }
}


sub delay_queue_retry {
    my $delaylist = shift;
    my %delaylist = %{$delaylist};
    my $retval;
    my $filesize;
    my $size_to_release = 0;
    my $messages_to_release = 0;
    my $dir;
    my $filename;
    my $filepath;
    my @gid;
    my $gid;


    for  $gid (keys %delaylist) {
	$dir = $delaylist{$gid};
	
	if ( ($gid =~ m/(\d+)\-(\d+)/) ) {
	    $filename = $1;
	} else {
	    $logger->log(LOG_ERR, "invalid $gid came from ui for release\n");
	    next;
	}
	$filepath = "$delay_queue_dir/$dir/$filename";
	if (-f "$filepath") {
	    $filesize = -s "$filepath";
	    $retval = delay_queue_retry_do($dir, $filename, $filepath);
	    if ($retval == 1) {
		$size_to_release += $filesize;
		$messages_to_release++;
		$logger->log(LOG_INFO, "delay file=$filename gid=$gid is marked for rescan from ui\n");
		#prepare list of gid to be removed;
                push @gid, $gid;
	    }
	}
    }
    if ($messages_to_release != 0) {
	set_delay_queue_size($size_to_release, $messages_to_release);
	#remove these records from db.
	eval {
	    my %opts=('logger'=>$logger);
	    my $pmdb= PureMessage::Policy::Delay::DelayqPmdb->new($qdirpath, %opts);
	    $pmdb->_trash_delayq_messages(\@gid);
	};
	if ($@) {
	     $logger->log(LOG_ERR, "_trash_delayq_messages failed for gid @gid, db can not be updated. err[$@]");
	}
    }
}


sub delay_queue_delete {
    my $delaylist = shift;
    my %delaylist = %{$delaylist};
    my $retval;
    my $filesize;
    my $filename;
    my $size_to_release = 0;
    my $messages_to_release = 0;
    my $dir;
    my @gid;
    my $gid;

    for $gid (keys %delaylist) {
	$dir = $delaylist{$gid};
	if ( ($gid =~ m/(\d+)\-(\d+)/) ) {
	    $filename = $1;
	} else {
	    $logger->log(LOG_ERR, "invalid $gid came from ui for delete\n");
	    next;
	}
	$filename = "$delay_queue_dir/$dir/$filename";
	if ( -f $filename) {
	    $filesize = -s $filename;
	    if (unlink ($filename)) {
		$size_to_release += $filesize;
		$messages_to_release++;

		$logger->log(LOG_INFO, "delay file=$filename gid=$gid is deleted from ui\n");
		#prepare list of gid to be removed;
                push @gid, $gid;
	    }
	}
    }
    if ($messages_to_release != 0) {
	set_delay_queue_size($size_to_release, $messages_to_release);
	#remove these records from db.
	eval {
	    my %opts=('logger'=>$logger);
	    my $pmdb= PureMessage::Policy::Delay::DelayqPmdb->new($qdirpath, %opts);
	    $pmdb->_trash_delayq_messages(\@gid);
	};
	if ($@) {
	     $logger->log(LOG_ERR, "_trash_delayq_messages failed for gid @gid, db can not be updated. err[$@]");
	}
    }
}
