package PureMessage::Policy::Delay::RCWrapper;

use strict;
use warnings;
use Redis;

use PureMessage::Version qw($PREFIX);
use PureMessage::_sys;
use PureMessage::Policy::Delay::Redis::Config qw(%redis_config);

our $VERSION = '6.4';
our (@ISA, @EXPORT );
BEGIN {
	require Exporter;
	@ISA = qw(Exporter);
	@EXPORT    = qw($HDB_IS_UP);
}

our $HDB_IS_UP = 0;
my $shdb_client = undef;

# connect to sender history database
sub connect_shdb {
	return $shdb_client if $shdb_client;
	my $shdb_server_ip   = $redis_config{server_ip};
	my $shdb_server_port = $redis_config{port};
	my $shdb_id = 0;
    
	 my %redis_obj=(
	    server    => "$shdb_server_ip:$shdb_server_port",
		name      => PureMessage::_sys::gethostname() . ":$$",
		reconnect => 3,
		every     => 1.5 * 1000000,
		conservative_reconnect => 1,
		no_auto_connect_on_new => 1,
		cnx_timeout            => 15,
	);
	if (defined $redis_config{password} ){
		$redis_obj{password}=  $redis_config{password};
	}		
	$shdb_client = Redis->new(%redis_obj);
	eval {
		$shdb_client->connect;
		$HDB_IS_UP = 1;
	};
	if ($@) {
		$HDB_IS_UP = 0;
		$shdb_client = undef;
	}
	return $shdb_client;
}
connect_shdb();

sub new {
	my $class = shift;
	my $self = {};
	$self->{shdb_client} = $shdb_client;
	bless $self, $class;
	return $self;
}

sub AUTOLOAD {
	return unless $HDB_IS_UP;
	my $self = shift;
	our $AUTOLOAD;
	my $method = $AUTOLOAD;
	# parse method name
	$method =~ s/.*::(_?)//;
	my $result = undef;
	
	eval {
		$result = $shdb_client->$method(@_);
	};
	if($@) {
		my $error_msg = chomp($@);
		open(my $log, '>>', "$PREFIX/var/log/redis.log");
		print $log "WARNING: Unable to execute redis command $method\nerror: $error_msg\n";
		close($log);
	}
	return $result;
}

1;

