package PureMessage::Policy::Delay::Redis::Config;

use strict;
use warnings;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = '%redis_config';
our %redis_config;

# Validation rules and default values.
my %conf = (
	server_ip	=> ['#str', '127.0.0.1'],
	port		=> ['#num', 6379],
	password	=> ['#str',undef],
);

refresh();

sub new {
	my $self = shift;
	my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
	PureMessage::Policy::Delay::Redis::Config->validate($p);
	$p->parse_file(expand_filename("etc/redis_location.conf"));
	$p->as_hash;
	$p;
}

sub refresh {
	%redis_config = new(undef)->as_hash;
}

sub validate {
	my ($pkg, $p) = @_;

	# Add our validation rules to the parser.
	foreach my $key (keys %conf) {
		my ($type, $def) = @{$conf{$key}};
		$p->typedecl( $key, $type ) if (defined $type);
		$p->default(  $key, $def )  if (defined $def);
	}
}

1;
