package PureMessage::Policy::Delay::Service;
use strict;
use warnings;
use Carp;
use ActiveState::Handy qw(run);
use ActiveState::Unix::ProcInfo qw(proc_info);
use PureMessage::Version qw($PREFIX);
use PureMessage::Config qw(expand_filename);

use base 'Exporter';
our @EXPORT_OK = qw(
    dqr_pid
    dqr_is_running
    
    dqr_start
    dqr_stop
    dqr_restart
    dqr_status
);

my $pidfile = "$PREFIX/var/run/delay_queue.pid";
sub dqr_pid{
    my $pid_file = expand_filename($pidfile);
    my $pids = [];
    return 0 unless -e $pid_file;
	open(PF, '<', $pid_file) || die "Unable to open file $pid_file $!";
	while (my $pid = <PF>) {
		chomp($pid);
		push @$pids, $pid;
	}
	close PF;
    return $pids;
}

sub dqr_is_running{
    my $pids = dqr_pid();
    return 0 unless $pids;
	# For Delay Queue Runner will check only master process's status and ignore child
    my $info = proc_info ('root_pid' => $pids->[0]);
    return 0 unless $info && $$info{$pids->[0]}; 
    return 1;
}

sub dqr_start{
    return 1 if dqr_is_running();

    my $cmd  = expand_filename('bin/delay_queue_runner');
    unless( -x $cmd) {
        croak('delay-queue-runner program does not have permissions to run.');
    }

    my $rc = 1;
    eval {$rc = run("\@$cmd >/dev/null 2>&1"); };
	return 0 if ($rc || $@);

    # Poll for delay-queue-runner to start.
    my $count = 10;
    while ($count--) {
        sleep(1);
        if (dqr_is_running()) {
          return 1;
        }
    }

    return 0;
}

sub dqr_stop{
    return 1 unless dqr_is_running();
    my $pids = dqr_pid();
	return unless $pids;
	foreach my $pid ( @{$pids} ) {
		my $count = 30;
		if(kill(0, $pid)){
			kill('TERM', $pid);
			while($count and kill(0,$pid)) {
			  $count--;  
			  sleep(1);
			}
			return $count;
		}
	}
    return 0;
}

sub dqr_restart{
  my $c = 1;
  if(dqr_is_running()){
    $c = dqr_stop();
  }
  return 0 unless $c;
  return dqr_start();
}

sub dqr_status{
  if (dqr_is_running()) {
    print "pmx-delay-queue-runner:\n";
	my $pids = dqr_pid();
	my $pid_str = join( ', ' , @{$pids} );
    print "\tpids :     $pid_str\n";
    return 1;
  } else {
    print "pmx-delay-queue-runner is not running.\n";
    return 0;
  }
}

1;

